@@ -125,7 +125,7 @@ class CFunction(CConstruct):  # pylint:disable=abstract-method
     def __init__(self, name, functy: SimTypeFunction, arg_list: List['CExpression'], statements, variables_in_use,
                  variable_manager, demangled_name=None):
 
-        super(CFunction, self).__init__()
+        super().__init__()
 
         self.name = name
         self.functy = functy
@@ -141,6 +141,9 @@ class CFunction(CConstruct):  # pylint:disable=abstract-method
 
         # output each variable and its type
         for var, cvar in self.variables_in_use.items():
+            if cvar in self.arg_list:
+                continue
+
             var_type = self.variable_manager.get_variable_type(var)
 
             if var_type is None:
@@ -245,7 +248,7 @@ class CStatements(CStatement):
 
     def __init__(self, statements):
 
-        super(CStatements, self).__init__()
+        super().__init__()
 
         self.statements = statements
 
@@ -264,7 +267,7 @@ class CAILBlock(CStatement):
 
     def __init__(self, block):
 
-        super(CAILBlock, self).__init__()
+        super().__init__()
 
         self.block = block
 
@@ -295,7 +298,7 @@ class CWhileLoop(CLoop):
 
     def __init__(self, condition, body):
 
-        super(CWhileLoop, self).__init__()
+        super().__init__()
 
         self.condition = condition
         self.body = body
@@ -359,7 +362,7 @@ class CIfElse(CStatement):
 
     def __init__(self, condition, true_node=None, false_node=None):
 
-        super(CIfElse, self).__init__()
+        super().__init__()
 
         self.condition = condition
         self.true_node = true_node
@@ -403,7 +406,7 @@ class CIfBreak(CStatement):
 
     def __init__(self, condition):
 
-        super(CIfBreak, self).__init__()
+        super().__init__()
 
         self.condition = condition
 
@@ -502,7 +505,7 @@ class CAssignment(CStatement):
 
     def __init__(self, lhs, rhs):
 
-        super(CAssignment, self).__init__()
+        super().__init__()
 
         self.lhs = lhs
         self.rhs = rhs
@@ -594,9 +597,9 @@ class CReturn(CStatement):
 
         indent_str = self.indent_str(indent=indent)
 
-        if self.retval is None:
+        if not self.retval:
             yield indent_str, None
-            yield "return;", None
+            yield "return;\n", None
         else:
             yield indent_str, None
             yield "return ", None
@@ -664,6 +667,19 @@ class CStructField(CExpression):
         yield str(self.field), self
 
 
+class CPlaceholder(CExpression):
+
+    __slots__ = ('placeholder', )
+
+    def __init__(self, placeholder):
+        super().__init__()
+
+        self.placeholder: str = placeholder
+
+    def c_repr_chunks(self):
+        yield self.placeholder, self
+
+
 class CVariable(CExpression):
     """
     Read value from a variable.
@@ -695,7 +711,10 @@ class CVariable(CExpression):
     def c_repr_chunks(self):
         if self.offset is None:
             if isinstance(self.variable, SimVariable):
-                yield str(self.variable.name), self
+                if self.variable.name:
+                    yield self.variable.name, self
+                else:
+                    yield str(self.variable), self
             elif isinstance(self.variable, CExpression):
                 if isinstance(self.variable, CVariable) and self.variable.type is not None:
                     if isinstance(self.variable.type, SimTypePointer):
@@ -862,6 +881,28 @@ class CBinaryOp(CExpression):
             return self.lhs.type
         return self._type
 
+    @property
+    def op_precedence(self):
+        precedence_list = [
+            # lowest precedence
+            ['Concat'],
+            ['LogicalOr'],
+            ['LogicalAnd'],
+            ['Or'],
+            ['Xor'],
+            ['And'],
+            ['CmpEQ', 'CmpNE'],
+            ['CmpLE', 'CmpLT', 'CmpGT', 'CmpGE'],
+            ['Shl', 'Shr', 'Sar'],
+            ['Add', 'Sub'],
+            ['Mul', 'Div'],
+            # highest precedence
+        ]
+        for i, sublist in enumerate(precedence_list):
+            if self.op in sublist:
+                return i
+        return len(precedence_list)
+
     def c_repr_chunks(self):
 
         if self.variable is not None:
@@ -883,11 +924,16 @@ class CBinaryOp(CExpression):
             'LogicalAnd': self._c_repr_chunks_logicaland,
             'LogicalOr': self._c_repr_chunks_logicalor,
             'CmpLE': self._c_repr_chunks_cmple,
+            'CmpLEs': self._c_repr_chunks_cmple,
             'CmpLT': self._c_repr_chunks_cmplt,
+            'CmpLTs': self._c_repr_chunks_cmplt,
             'CmpGT': self._c_repr_chunks_cmpgt,
+            'CmpGTs': self._c_repr_chunks_cmpgt,
             'CmpGE': self._c_repr_chunks_cmpge,
+            'CmpGEs': self._c_repr_chunks_cmpge,
             'CmpEQ': self._c_repr_chunks_cmpeq,
             'CmpNE': self._c_repr_chunks_cmpne,
+            'Concat': self._c_repr_chunks_concat,
         }
 
         handler = OP_MAP.get(self.op, None)
@@ -900,97 +946,80 @@ class CBinaryOp(CExpression):
     # Handlers
     #
 
+    def _c_repr_chunks(self, op):
+        # lhs
+        if isinstance(self.lhs, CBinaryOp) and self.op_precedence > self.lhs.op_precedence:
+            yield "(", None
+            yield from self._try_c_repr_chunks(self.lhs)
+            yield ")", None
+        else:
+            yield from self._try_c_repr_chunks(self.lhs)
+        # operator
+        yield op, None
+        # rhs
+        if isinstance(self.rhs, CBinaryOp) and self.op_precedence > self.rhs.op_precedence - (1 if self.op in ['Sub', 'Div'] else 0):
+            yield "(", None
+            yield from self._try_c_repr_chunks(self.rhs)
+            yield ")", None
+        else:
+            yield from self._try_c_repr_chunks(self.rhs)
+
     def _c_repr_chunks_add(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " + ", None
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" + ")
 
     def _c_repr_chunks_sub(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " - ", None
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" - ")
 
     def _c_repr_chunks_mul(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " * ", None
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" * ")
 
     def _c_repr_chunks_div(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " / ", None
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" / ")
 
     def _c_repr_chunks_and(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " & ", None
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" & ")
 
     def _c_repr_chunks_xor(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " ^ ", None
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" ^ ")
 
     def _c_repr_chunks_or(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " | ", None
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" | ")
 
     def _c_repr_chunks_shr(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " >> ", None
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" >> ")
 
     def _c_repr_chunks_shl(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " << ", None
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" << ")
 
     def _c_repr_chunks_sar(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " >> ", None
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" >> ")
 
     def _c_repr_chunks_logicaland(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " && ", None
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" && ")
 
     def _c_repr_chunks_logicalor(self):
-        yield "(", None
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield ") || (", None
-        yield from self._try_c_repr_chunks(self.rhs)
-        yield ")", None
+        yield from self._c_repr_chunks(" || ")
 
     def _c_repr_chunks_cmple(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " <= ", self
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" <= ")
 
     def _c_repr_chunks_cmplt(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " < ", self
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" < ")
 
     def _c_repr_chunks_cmpgt(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " > ", self
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" > ")
 
     def _c_repr_chunks_cmpge(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " >= ", self
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" >= ")
 
     def _c_repr_chunks_cmpeq(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " == ", self
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" == ")
 
     def _c_repr_chunks_cmpne(self):
-        yield from self._try_c_repr_chunks(self.lhs)
-        yield " != ", self
-        yield from self._try_c_repr_chunks(self.rhs)
+        yield from self._c_repr_chunks(" != ")
+
+    def _c_repr_chunks_concat(self):
+        yield from self._c_repr_chunks(" CONCAT ")
 
 
 class CTypeCast(CExpression):
@@ -1125,7 +1154,7 @@ class CDirtyExpression(CExpression):
 
 class StructuredCodeGenerator(Analysis):
     def __init__(self, func, sequence, indent=0, cfg=None, variable_kb=None,
-                 func_args: Optional[List[SimVariable]]=None):
+                 func_args: Optional[List[SimVariable]]=None, binop_depth_cutoff: int=10):
 
         self._handlers = {
             CodeNode: self._handle_Code,
@@ -1143,6 +1172,7 @@ class StructuredCodeGenerator(Analysis):
             Stmt.Assignment: self._handle_Stmt_Assignment,
             Stmt.Call: self._handle_Stmt_Call,
             Stmt.Jump: self._handle_Stmt_Jump,
+            Stmt.Return: self._handle_Stmt_Return,
             # AIL expressions
             Expr.Register: self._handle_Expr_Register,
             Expr.Load: self._handle_Expr_Load,
@@ -1165,8 +1195,10 @@ class StructuredCodeGenerator(Analysis):
         self._cfg = cfg
         self._sequence = sequence
         self._variable_kb = variable_kb if variable_kb is not None else self.kb
+        self.binop_depth_cutoff = binop_depth_cutoff
 
         self._variables_in_use: Optional[Dict] = None
+        self._memo: Optional[Dict[Expr,CExpression]] = None
 
         self.text = None
         self.posmap = None
@@ -1178,12 +1210,19 @@ class StructuredCodeGenerator(Analysis):
     def _analyze(self):
 
         self._variables_in_use = {}
+
+        # memo
+        self._memo = {}
+
         if self._func_args:
             arg_list = [self._handle(arg) for arg in self._func_args]
         else:
             arg_list = [ ]
+
         obj = self._handle(self._sequence)
 
+        self._memo = None  # clear the memo since it's useless now
+
         func = CFunction(self._func.name, self._func.prototype, arg_list, obj, self._variables_in_use,
                          self._variable_kb.variables[self._func.addr], demangled_name=self._func.demangled_name)
         self._variables_in_use = None
@@ -1267,12 +1306,19 @@ class StructuredCodeGenerator(Analysis):
     #
 
     def _handle(self, node, is_expr: bool=True):
+
+        if node in self._memo:
+            return self._memo[node]
+
         handler = self._handlers.get(node.__class__, None)
         if handler is not None:
             # special case for Call
             if isinstance(node, Stmt.Call):
-                return handler(node, is_expr=is_expr)
-            return handler(node)
+                converted = handler(node, is_expr=is_expr)
+            else:
+                converted = handler(node)
+            self._memo[node] = converted
+            return converted
         raise UnsupportedNodeTypeError("Node type %s is not supported yet." % type(node))
 
     def _handle_Code(self, node):
@@ -1488,6 +1534,22 @@ class StructuredCodeGenerator(Analysis):
     def _handle_Stmt_Jump(self, stmt):
         return CGoto(self._handle(stmt.target))
 
+    def _handle_Stmt_Return(self, stmt: Stmt.Return):
+        if not stmt.ret_exprs:
+            return CReturn(None)
+        elif len(stmt.ret_exprs) == 1:
+            ret_expr = stmt.ret_exprs[0]
+            if ret_expr.variable is not None:
+                return CReturn(self._cvariable(ret_expr.variable, offset=ret_expr.variable_offset))
+            return CReturn(self._handle(ret_expr))
+        else:
+            # TODO: Multiple return expressions
+            l.warning("StructuredCodeGen does not support multiple return expressions yet. Only picking the first one.")
+            ret_expr = stmt.ret_exprs[0]
+            if ret_expr.variable is not None:
+                return CReturn(self._cvariable(ret_expr.variable, offset=ret_expr.variable_offset))
+            return CReturn(self._handle(ret_expr))
+
     #
     # AIL expression handlers
     #
@@ -1540,6 +1602,9 @@ class StructuredCodeGenerator(Analysis):
 
     def _handle_Expr_BinaryOp(self, expr):
 
+        if expr.depth > self.binop_depth_cutoff:
+            return CPlaceholder("...")
+
         lhs = self._handle(expr.operands[0])
         rhs = self._handle(expr.operands[1])
         rhs.set_type(lhs.type)
