@@ -126,6 +126,11 @@ class SimEngineVRAIL(
     def _ail_handle_CallExpr(self, expr: ailment.Stmt.Call):
         return self._ail_handle_Call(expr, is_expr=True)
 
+    def _ail_handle_Return(self, stmt: ailment.Stmt.Return):
+        if stmt.ret_exprs:
+            for ret_expr in stmt.ret_exprs:
+                self._expr(ret_expr)
+
     # Expression handlers
 
     def _expr(self, expr):
@@ -503,6 +508,16 @@ class SimEngineVRAIL(
             self.l.warning("_ail_handle_Or(): TypeError.", exc_info=True)
             return RichR(None)
 
+    def _ail_handle_Concat(self, expr):
+
+        arg0, arg1 = expr.operands
+
+        _ = self._expr(arg0)
+        _ = self._expr(arg1)
+
+        # TODO: Model the operation. Don't lose type constraints
+        return RichR(None)
+
     def _ail_handle_Not(self, expr):
         arg = expr.operands[0]
         expr = self._expr(arg)
